import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import { writeFile } from 'node:fs/promises'
import path from 'path'
import fs from 'fs/promises'
import prisma from '../../../../utilities/prisma'
import CharacterManager from '../../../../managers/characterManager'

interface ITileData {
  [key: string]: Buffer
}

/**
 * Handle game master upload tile event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
  socket.on('gm:tile:upload', async (data: ITileData, callback: (response: boolean) => void) => {
    try {
      const character = CharacterManager.getCharacterFromSocket(socket);
      if (character?.role !== 'gm') {
        callback(false)
        return
      }

      const public_folder = path.join(process.cwd(), 'public', 'tiles')

      // Ensure the folder exists
      await fs.mkdir(public_folder, { recursive: true })

      const uploadPromises = Object.entries(data).map(async ([key, tileData]) => {
        const tile = await prisma.tile.create({
          data: {
            name: 'New tile'
          }
        })
        const uuid = tile.id
        const filename = `${uuid}.png`
        const finalFilePath = path.join(public_folder, filename)
        await writeFile(finalFilePath, tileData)
      })

      await Promise.all(uploadPromises)

      callback(true)
    } catch (error) {
      console.error('Error uploading tile:', error)
      callback(false)
    }
  })
}
