import { Server } from 'socket.io'
import { TSocket } from '../../../utilities/types'
import ZoneRepository from '../../../repositories/zoneRepository'
import { Zone } from '@prisma/client'
import CharacterManager from '../../../managers/characterManager'

interface IPayload {
  zoneId: number
}

/**
 * Handle game master zone request event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
  socket.on('gm:zone_editor:zone:request', async (data: IPayload, callback: (response: Zone) => void) => {
    const character = CharacterManager.getCharacterFromSocket(socket);
    if (character?.role !== 'gm') {
      return
    }

    console.log(`---GM ${character?.id} has requested zone via zone editor.`)

    if (!data.zoneId) {
      console.log(`---Zone id not provided.`)
      return
    }

    try {
      const zone = await ZoneRepository.getById(data.zoneId)

      if (!zone) {
        console.log(`---Zone not found.`)
        return
      }

      callback(zone)
    } catch (e) {
      console.error(e)
    }
  })
}
