import { Server } from 'socket.io'
import { ExtendedCharacter, TSocket } from '../../utilities/types'
import ZoneRepository from '../../repositories/zoneRepository'
import ZoneManager from '../../managers/zoneManager'
import { Character, Zone } from '@prisma/client'
import logger from '../../utilities/logger'
import CharacterManager from '../../managers/characterManager'

interface IPayload {
  zoneId: number
}

interface IResponse {
  zone: Zone
  characters: Character[]
}

/**
 * Handle character zone request event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
  socket.on('zone:character:join', async (data: IPayload, callback: (response: IResponse) => void) => {
    try {
      if (!socket.characterId) return
      const character = CharacterManager.getCharacterFromSocket(socket);

      if(!character) return

      if (!data.zoneId) {
        console.log(`---Zone id not provided.`)
        return
      }

      const zone = await ZoneRepository.getById(data.zoneId)

      if (!zone) {
        console.log(`---Zone not found.`)
        return
      }

      if (character?.zoneId) {
        socket.leave(character.zoneId.toString())
        io.to(character.zoneId.toString()).emit('zone:character:leave', character)
      }

      socket.join(zone.id.toString())

      // let other clients know of new character
      io.to(zone.id.toString()).emit('zone:character:join', character)

      // add character to zone manager
      // ZoneManager.addCharacterToZone(zone.id, socket.character as Character)
      // CharacterManager.initCharacter(character as ExtendedCharacter)
      // ZoneManager.addCharacterToZone(zone.id, socket.character as Character)

      // send over zone and characters to socket
      callback({ zone, characters: CharacterManager.getCharactersInZone(zone) })
    } catch (error: any) {
      logger.error(`Error requesting zone: ${error.message}`)
      socket.disconnect()
    }
  })
}
